import logging
import time
import random

import requests

from .._base_dler import BaseDler
from ..converter import apple_music

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class AudioDler(BaseDler):
    AUDIO_API = 'https://api.bilibili.com/audio/music-service-c/url'

    def __init__(self, sessdata, dl_path=None, cfg_path=None, music=True, dry_run=False):
        self.apple_music = music
        super().__init__(sessdata=sessdata, dl_path=dl_path, cfg_path=cfg_path, mode='audio', dry_run=dry_run)

    def __call__(self, *args, **kwargs):
        items = self.parser(*args, **kwargs)
        audios = self.download_audios(items['audio'])
        if self.apple_music and not self.dry_run:
            files = [apple_music(audio=audio['audio'], music_path=self.dl_path, metadata=audio['metadata'],
                                 lyric=audio['lyric'], cover=audio['cover']) for aid, audio in audios.items()]
            return files
        return audios

    def download_audios(self, items):
        files = {}
        for aid in items:
            param = {'songid': aid['id'], 'quality': 3, 'privilege': 2, 'mid': '', 'platform': ''}
            a_info = requests.get(self.AUDIO_API, params=param, cookies=self.cookie).json()['data']
            logging.info('Downloading Audio:', aid['title'])
            files[aid['id']] = {'cover': self._download(aid['cover']), 'lyric': self._download(aid['lyric']),
                                'audio': self.download(a_info['data']),
                                'metadata': {'\xa9nam': aid['title'], '\xa9alb': aid['title'], '\xa9ART': aid['uname'],
                                             'aART': aid['uname'], '\xa9day': time.localtime(aid['passtime']).tm_year,
                                             '\xa9cmt': aid['intro']}, 'desc': ''}
            time.sleep(random.random())
        return files



import os.path
import time
import random
import logging
import requests

from .._base_dler import BaseDler
from ..converter import merge as _merge

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class VideoDler(BaseDler):
    VIDEO_API = 'https://api.bilibili.com/x/player/playurl'
    page_param = {'cid': None, 'fnval': 16}

    def __init__(self, sessdata, dl_path=None, cfg_path=None, merge=True, video=True, audio=True, dry_run=False):
        self.merge = merge
        self.video = video
        self.audio = audio
        super().__init__(sessdata=sessdata, dl_path=dl_path, cfg_path=cfg_path, mode='video', dry_run=dry_run)

    def __call__(self, *args, **kwargs):
        items = self.parser(*args, **kwargs)
        videos = self.download_videos(items['video'], video=self.video, audio=self.audio)
        if self.merge and not self.dry_run:
            files = [_merge(os.path.join(self.dl_path, f'{video}_{page}.mov'), video=info['video'], audio=info['audio'])
                     for video, pages in videos.items() for page, info in pages.items()]
            return files
        return videos
    
    def download_videos(self, items, video=True, audio=True):
        files = {}
        for v_info in items:
            self.page_param.update({'bvid': v_info['bvid']})
            logging.info('Downloading Video: %s', v_info['title'])
            files[v_info['bvid']] = {'cover': self._download(v_info['pic'])}
            for page in v_info['pages']:
                logging.info('\tDownloading Page: %s', page['part'])
                self.page_param['cid'] = page['cid']
                p_info = requests.get(self.VIDEO_API, params=self.page_param, cookies=self.cookie).json()['data']
                staffs = {'UP主': [staff['name'] for staff in v_info['staff'] if staff['title'] == 'UP主'][0],
                          '演唱': [staff['name'] for staff in v_info['staff'] if staff['title'] == '演唱']}
                file = {'metadata': {'\xa9nam': page['part'], '\xa9alb': v_info['title'], 'aART': staffs['UP主'],
                                     '\xa9ART': ';'.join([staffs['UP主']] + staffs['演唱']), '\xa9cmt': v_info['desc'],
                                     '\xa9day': time.localtime(v_info['pubdate']).tm_year, 'desc': ''}}
                if video:
                    file['video'] = self.download(p_info['dash']['video'][0]['baseUrl'])
                    time.sleep(random.random())
                if audio:
                    file['audio'] = self.download(p_info['data']['audio'][0]['baseUrl'])
                    time.sleep(random.random())
                files[v_info['title']][page['part']] = file
        return files



import subprocess


def merge(output, video=None, audio=None):
    cmd = ['ffmpeg']
    if video:
        cmd.extend(['-i', video])
    if audio:
        cmd.extend(['-i', audio])
    cmd.extend(['-c', 'copy', '-y', output])
    subprocess.run(cmd)
    return output



pics = {}
with open('train_list.txt') as f:
    for line in f.readlines():
        img, pid = line.rstrip().split()
        if pid in pics.keys():
            pics[pid].append(img)
        else:
            pics[pid] = [img]

avl = [len(v) for k, v in pics.items()]
cnt_old = sum(avl)
avg = int(cnt_old / len(avl))
cnt = 0
for k, v in pics.items():
    pics[k] = v[:avg]
    cnt += len(pics[k])

result = []
for k, v in pics.items():
    for p in v:
        result.append(f'{p} {k}\n')

with open('new_train_list.txt', 'w') as f:
    f.writelines(result)



import os
import sqlite3
import tempfile
import shutil
from pathlib import Path
from urllib.parse import urlparse
from configparser import ConfigParser

import requests
from tqdm.auto import tqdm

from .parser import Parser


class BaseDler:
    HEADER = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) '
                            'AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15',
              'Referer': 'https://www.bilibili.com'}

    def __init__(self, sessdata: str, dl_path=None, cfg_path=None, mode='', dry_run=False):
        """

        :param sessdata: Bilibili Login Cookie
        :param dl_path:
        :param cfg_path:
        """
        self.parser = Parser(sessdata=sessdata, mode=mode)
        self.cookie = {'SESSDATA': sessdata}
        self.dl_path = Path(dl_path) if dl_path else Path.cwd()
        self.cfg, self.cfg_path = self.init_config(cfg_path)
        self.tmpdir = tempfile.TemporaryDirectory()
        self.dry_run = dry_run
        self.db = self.init_history()

    def init_history(self):
        db = sqlite3.connect(os.path.join(self.cfg_path, 'history.sqlite'))
        db.execute('CREATE TABLE IF NOT EXISTS video (bv_id string PRIMARY KEY)')
        db.execute('CREATE TABLE IF NOT EXISTS audio (audio_id string PRIMARY KEY)')
        db.execute('CREATE TABLE IF NOT EXISTS album (doc_id string PRIMARY KEY)')
        db.commit()
        return db

    @staticmethod
    def init_config(cfg_path):
        cfg_path = Path(cfg_path) if cfg_path else os.path.join(Path.cwd(), '.bili_dler')
        os.makedirs(cfg_path, exist_ok=True)
        cfg = ConfigParser()
        cfg.read(os.path.join(cfg_path, 'config.ini'))
        return cfg, cfg_path

    def __del__(self):
        self.tmpdir.cleanup()
        self.db.close()

    def download(self, url):
        if self.dry_run:
            return 0
        fp = os.path.join(self.tmpdir.name, urlparse(url).path.split('/')[-1])
        with (requests.get(url, headers=self.HEADER, cookies=self.cookie, stream=True) as r,
              tqdm.wrapattr(r.raw, "read", total=int(r.headers.get("Content-Length")), desc="") as raw,
              open(fp, mode='wb') as f):
            shutil.copyfileobj(raw, f)
        return fp

    def _download(self, url):
        if self.dry_run:
            return 0
        fp = os.path.join(self.tmpdir.name, urlparse(url).path.split('/')[-1])
        with open(fp, mode='wb') as f:
            f.write(requests.get(url).content)
        return fp

    def examine_history(self, items):
        selected = {'video': [], 'audio': [], 'album': []}
        for item in items['video']:
            bv_id = list(item.values())[0]
            if len(self.db.execute('SELECT * FROM video WHERE bv_id=?', (bv_id, )).fetchall()) == 0:
                selected['video'].append(item)
                self.db.execute('INSERT INTO video VALUES (?)', (bv_id, ))
                self.db.commit()
        for item in items['audio']:
            if len(self.db.execute('SELECT * FROM audio WHERE audio_id=?', (item['id'], )).fetchall()) == 0:
                selected['audio'].append(item)
                self.db.execute('INSERT INTO audio VALUES (?)', (item['id'],))
                self.db.commit()
        for item in items['album']:
            if len(self.db.execute('SELECT * FROM album WHERE doc_id=?', (item['doc_id'], )).fetchall()) == 0:
                selected['album'].append(item)
                self.db.execute('INSERT INTO audio VALUES (?)', (item['doc_id'],))
                self.db.commit()
        return selected



import requests
import json

from .._base_dler import BaseDler

HEADER['Content-Type'] = 'application/json'
BUY_API = 'https://manga.bilibili.com/twirp/user.v1.User/GetAutoBuyComics'


def get_comics(cookie) -> []:
    comics = []
    cookie['SESSDATA'] = 'e36f978d%2C1672999949%2C1f56d*71'
    data = {"page_num": 1,"page_size": 100}
    while True:
        req = requests.post(BUY_API, data=json.dumps(data), headers=HEADER, cookies=cookie).json()
        comics.extend(req['data'])
        data["page_num"] += 1
        if len(req['data']) < 1:
            break
    return comics



from .bilibili_downloader import BiliDownloader, main
from .downloader import VideoDler, AudioDler, AlbumDler, MangaDler, UpDler

__all__ = {'BiliDownloader', 'VideoDler', 'AudioDler', 'AlbumDler', 'MangaDler', 'UpDler', 'main'}



import os
import cv2 as cv
from mutagen.mp4 import MP4, MP4Cover

from ._converter import merge


def apple_music(audio, music_path, metadata=None, cover=None, lyric=None):
    music_path = merge(output=os.path.join(music_path, metadata['\xa9nam'] + '.m4a'), audio=audio)

    with MP4(music_path) as fm:
        fm.update(metadata)
        if lyric:
            with open(lyric) as f:
                audio['\xa9lyr'] = f.read()
        if cover:
            img = cv.imread(cover)
            h, w, _ = img.shape
            if h % 2 == 1:
                h -= 1
            if w % 2 == 1:
                w -= 1
            cv.imwrite(cover, img[:h, :w])
            with open(cover, "rb") as f:
                fm["covr"] = [MP4Cover(f.read(), imageformat=MP4Cover.FORMAT_PNG)]
    fm.save()



import os
import logging

from .._base_dler import BaseDler
from ..converter import merge as _merge, apple_music
from .video import VideoDler
from .audio import AudioDler
from .albums import AlbumDler

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class UpDler(BaseDler):
    def __init__(self, sessdata, dl_path=None, cfg_path=None,
                 merge=True, album=True, video=True, audio=True, music=True, dry_run=False):
        self.video = VideoDler(sessdata, dl_path, cfg_path, dry_run=dry_run) if video else None
        self.merge = merge
        self.audio = AudioDler(sessdata, dl_path, cfg_path, dry_run=dry_run) if audio else None
        self.album = AlbumDler(sessdata, dl_path, cfg_path, dry_run=dry_run) if album else None
        self.apple_music = music

        super().__init__(sessdata=sessdata, dl_path=dl_path, cfg_path=cfg_path, mode='up', dry_run=dry_run)

    def __call__(self, *args, **kwargs):
        items = self.parser(*args, **kwargs)
        files = {}
        if self.video:
            videos = self.video.download_videos(items['video'], video=bool(self.video), audio=bool(self.audio))
            files['video'] = [_merge(os.path.join(self.dl_path, f'{video}_{page}.mov'), video=info['video'],
                                     audio=info['audio']) for video, pages in videos.items()
                              for page, info in pages.items()] if self.merge and not self.dry_run else videos
        if self.audio:
            audios = self.audio.download_audios(items['audio'])
            files['audio'] = [apple_music(audio=audio['audio'], music_path=self.dl_path, metadata=audio['metadata'],
                                          lyric=audio['lyric'], cover=audio['cover'])
                              for aid, audio in audios.items()] if self.apple_music and not self.dry_run else audios
        if self.album:
            files['albums'] = self.album.download_albums(items['album'])

        return files



import math
import random
import time
import json
import logging
from urllib.parse import urlparse

import requests

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class Parser:
    API = {'nav': 'https://api.bilibili.com/x/space/navnum',
           'video': 'https://api.bilibili.com/x/space/arc/search',
           'audio': 'https://api.bilibili.com/audio/music-service/web/song/upper',
           'album': 'https://api.bilibili.com/x/dynamic/feed/draw/doc_list',
           'dynamic': 'https://api.vc.bilibili.com/dynamic_svr/v1/dynamic_svr/get_dynamic_detail'}
    AUDIO_API = 'https://api.bilibili.com/audio/music-service-c/web/song/info'
    INFO_API = 'https://api.bilibili.com/x/web-interface/view'
    header = {'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) '
                            'AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15'}
    param = {'video': {'mid': int, 'ps': 50, 'tid': 0, 'pn': 1, 'keyword': '', 'order': 'pubdate',
                       'order_avoid': 'true', 'jsonp': 'jsonp'},
             'audio': {'uid': int, 'pn': 1, 'ps': 10000, 'order': 1, 'jsonp': 'jsonp'},
             'album': {'uid': int, 'page_num': 0, 'page_size': 2147483647, 'biz': 'all', 'jsonp': 'jsonp'}}

    def __init__(self, sessdata, mode='up'):
        self.cookie = {'SESSDATA': sessdata}
        if mode == 'up':
            self.mode = ['video', 'audio', 'album', 'dynamic']
        elif mode == 'music':
            self.mode = ['video', 'audio']
        else:
            self.mode = mode

    def __call__(self, *args, **kwargs) -> dict:
        """

        :param args: [url], url, (url,), [uid], uid
        :param kwargs:
        :return:
        """
        self.results = {k: [] for k in self.mode}
        for arg in args:
            if isinstance(arg, (str, int)):
                arg = [arg]
            for ar in arg:
                if isinstance(ar, int):
                    self.uid_parser(arg)
                elif isinstance(ar, str):
                    url = urlparse(ar)
                    if url.scheme:
                        self.urlparser(url)
                    elif url.path:
                        self.uid_parser(arg)
                else:
                    raise SyntaxWarning
        return self.results

    def urlparser(self, url):
        logging.debug('urlparser')
        _id = url.path.split('/')[-1]
        if _id.startswith('BV') and 'video' in self.mode:
            param = {'bvid': _id}
            self.results['video'].append(requests.get(self.INFO_API, params=param, cookies=self.cookie).json()['data'])
        elif _id.startswith('av') and 'video' in self.mode:
            param = {'avid': _id}
            self.results['video'].append(requests.get(self.INFO_API, params=param, cookies=self.cookie).json()['data'])
        elif _id.startswith('au') and 'audio' in self.mode:
            self.results['audio'].append(requests.get(self.AUDIO_API, params={'sid': _id}).json()['data'])
        elif _id.isdigit() and 'dynamic' in self.mode:
            card = requests.get(self.API['dynamic'], params={'dynamic_id': _id}).json()['data']['card']['card']
            self.results['dynamic'].append(json.loads(card)['item'])
        else:
            raise SyntaxWarning

    def uid_parser(self, uid):
        logging.debug('_uid_parser')
        nav = requests.get(self.API['nav'], headers=self.header, params={'mid': uid}).json()
        if 'video' in self.mode:
            self.param['video']['mid'] = uid
            self.results['video'].extend(self.get_list(mode='video', num=nav['data']['video']))
        if 'audio' in self.mode:
            self.param['audio']['uid'] = uid
            self.results['audio'].extend(self.get_list(mode='audio', num=nav['data']['audio']))
        if 'album' in self.mode:
            self.param['album']['uid'] = uid
            self.results['album'].extend(self.get_list(mode='album', num=nav['data']['album']))

    def get_list(self, mode, num):
        li = []
        num = math.ceil(num / self.param[mode]['page_size' if mode == 'album' else 'ps'])
        for pn in range(1, num + 1):
            self.param[mode]['pn'] = pn
            req = requests.get(self.API[mode], headers=self.header, params=self.param[mode]).json()
            if mode == 'video':
                for v in req['data']['list']['vlist'][:2]:
                    li.append(requests.get(self.INFO_API, params={'bvid': v['bvid']}, cookies=self.cookie).json()['data'])
                    time.sleep(random.random())
            elif mode == 'audio':
                li.extend(req['data']['data'])
            elif mode == 'album':
                li.extend(req['data']['items'])
            time.sleep(random.random())
        return li


if __name__ == '__main__':
    alurl = 'https://t.bilibili.com/730581161356558377?spm_id_from=333.999.0.0'
    aurl = 'https://www.bilibili.com/audio/au2515961?type=3&spm_id_from=333.999.0.0'
    vurl = 'https://www.bilibili.com/video/BV16R4y1M7XC?from=search&seid=772507573660196375&spm_id_from=333.337.0.0'
    # Up(uid=353837540)
    p = Parser(sessdata='b4f0af61%2C1659237450%2C8f896*21', mode='up')
    p(353837540)
    i = 0



import os
import logging
import tempfile

from .converter import apple_music, merge
from ._base_dler import BaseDler
from .downloader import Video, Audio, Album, Dynamic, Manga

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class BiliDownloader(BaseDler):
    def __init__(self, sessdata: str, mode, dl_path=None, cfg_path=None, dry_run=False):
        """

        :param sessdata: Bilibili Login Cookie
        :param dl_path:
        :param cfg_path:
        """
        super().__init__(sessdata, dl_path, cfg_path)
        self.mode = self.parse_mode(mode)

    def __call__(self, *args, **kwargs):
        items = self.parser(*args, **kwargs)

        # with open('parse.json') as f:
        #     import json
        #     items = json.load(f)
        items = self.examine_history(items)

        with tempfile.TemporaryDirectory() as tmpdir:
            if 'video' in self.mode:
                videos = download_videos(items['video'], tmpdir, self.cookie, video=video, audio=audio)
                for k, v in videos.items():
                    merge(v['video'], v['audio'], os.path.join(self.dl_path, v['metadata']['\xa9nam'] + '.mov'))
            if audio:
                audios = download_audios(items['audio'], tmpdir, self.cookie)
            if album:
                albums = download_albums(items['album'], tmpdir)

            if music:
                for k, v in videos.items():
                    if k != 'cover':
                        fp = os.path.join(self.dl_path, v['metadata']['\xa9nam'] + '.m4a')
                        apple_music(v['audio'], fp, metadata=v['metadata'], cover=videos['cover'])
                for k, a in audios.items():
                    fp = os.path.join(self.dl_path, a['metadata']['\xa9nam'] + '.m4a')
                    apple_music(a['audio'], fp, metadata=a['metadata'], cover=a['cover'], lyric=a['lyric'])

    @staticmethod
    def parse_mode(mode: [list, tuple, str]):
        if isinstance(mode, str):
            mode = mode.split(',')
        if isinstance(mode, (list, tuple)):
            mode = set(mode)
            if 'up' in mode:
                mode.add(['video', 'audio', 'album'])
            if 'music' in mode:
                mode.add(['audio', 'video'])
            return mode
        else:
            raise TypeError('mode must be list, tuple or str')


def main():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('sessdata', help='Bilibili Login Cookie')
    parser.add_argument('mode', help='Download Mode')
    parser.add_argument('dl_path', help='Download Path')
    parser.add_argument('cfg_path', help='Config Path')
    args = parser.parse_args()
    b = BiliDownloader(args.sessdata, args.mode, args.dl_path, args.cfg_path)
    b()

if __name__ == '__main__':
    main()



import logging

from tqdm.auto import tqdm

from .._base_dler import BaseDler

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class AlbumDler(BaseDler):
    def __init__(self, sessdata: str, dl_path=None, cfg_path=None, dry_run=False):
        super().__init__(sessdata, dl_path, cfg_path, mode='albums', dry_run=dry_run)

    def __call__(self, *args, **kwargs):
        items = self.parser(*args, **kwargs)
        return self.download_albums(items['albums'])

    def download_albums(self, albums):
        files = {}
        for album in albums:
            logging.info('Downloading Albums: %s', album['doc_id'])
            files[album['doc_id']] = [self.download(picture['img_src']) for picture in tqdm(album['pictures'])]
        return files



from setuptools import setup

setup(
    name='BiliDownloader',
    version='0.0.4',
    packages=['src', 'src.bili_downloader'],
    url='https://github.com/nkjcqvcpi/BiliDownloader',
    license='GPL',
    author='Nkjcqvcpi',
    author_email='lhtt100@gmail.com',
    description='A Downloader for Bili',
)



from src.bili_downloader import BiliDownloader, VideoDler, UpDler


if __name__ == '__main__':
    # d = VideoDler(sessdata='b4f0af61%2C1659237450%2C8f896*21', dl_path='/Volumes/THL/影片', dry_run=True)
    # x = d('https://www.bilibili.com/video/BV1J4411v7g6')
    d = UpDler(sessdata='b4f0af61%2C1659237450%2C8f896*21', dl_path='/Volumes/THL/影片', dry_run=True)
    # d = BiliDownloader(sessdata='b4f0af61%2C1659237450%2C8f896*21', dl_path='/Volumes/THL/影片')
    d(353837540)
    # d(['https://www.bilibili.com/video/BV1J4411v7g6']) # 353837540

    i = 0



import json
import logging

import requests
from .._base_dler import BaseDler

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


MAN_API = 'https://manga.bilibili.com/twirp/comic.v1.Comic/ComicDetail'
INDEX_API = 'https://manga.bilibili.com/twirp/comic.v1.Comic/GetImageIndex'
TOKEN_API = 'https://manga.bilibili.com/twirp/comic.v1.Comic/ImageToken'


class MangaDler(BaseDler):
    def __init__(self):
        self.HEADER['Content-Type'] = 'application/json'

    def download_manga(self, comic_id, tmpdir, cookie):
        mg = requests.post(MAN_API, data=json.dumps({"comicId": comic_id}), headers=self.HEADER, params={'device': 'pc'}).json()
        manga = [download_episode(ep['id'], tmpdir, cookie) for ep in mg['ep_list']]
        return manga

    def download_episode(self, ep_id, tmpdir, cookie):
        cookie['SESSDATA'] = 'e36f978d%2C1672999949%2C1f56d*71'
        idx = requests.post(INDEX_API, data=json.dumps({"ep_id": ep_id}), cookies=cookie, headers=HEADER,
                            params={'device': 'pc'}).json()

        urls = '["' + '","'.join([f'https://i0.hdslb.com{img["path"]}' for img in idx['data']['images']]) + '"]'

        req = requests.post(TOKEN_API, data=json.dumps({"urls": urls}), cookies=cookie, headers=HEADER).json()
        imgs = [_download(img['url'] + '?token=' + img['token'], tmpdir) for img in req['data']]
        return imgs
